display.setDefault("background", 1)
display.setStatusBar(display.HiddenStatusBar)
transition.ignoreEmptyReference = true
if audio.supportsSessionProperty then
    audio.setSessionProperty(audio.MixMode, audio.AmbientMixMode) -- 게임에서 음악 재생 멈추지 않는 모드
end

orgW = display.actualContentWidth
orgH = display.actualContentHeight
centerX = display.contentCenterX
centerY = display.contentCenterY
safeW = display.safeActualContentWidth
safeH = display.safeActualContentHeight
emptyH = 0

if orgH-safeH > 200 then
    emptyH = math.abs( (safeH-orgH)/2 )
end

local widget = require "widget"

------------------------------------------------------------------
-- newGroup
------------------------------------------------------------------
function newGroup( parent )
    local g = display.newGroup()
    if parent then
        parent:insert(g)
    end
    return g
end

------------------------------------------------------------------
-- newRect
------------------------------------------------------------------
function newRect( parent, x, y, w, h, color, ref )
    local rect = display.newRect(x, y, w, h)

    if parent then
        parent:insert(rect)
    end
    
    if color then
        rect.fill = color
    end

    return rect
end

------------------------------------------------------------------
-- newText
------------------------------------------------------------------
function newText( parent, text, x, y, w, h, font, fontSize, color, align )
    local option = 
    {
        text = text,
        x = x,
        y = y,
        width = w,
        height = h,
        font = font,
        fontSize = fontSize,
        align = align,
    }

    local txt = display.newText( option )
    
    if parent then
        parent:insert(txt)
    end

    if color then
        txt.fill = color
    end

    return txt
end

------------------------------------------------------------------
-- newButton
------------------------------------------------------------------
function newButton( parent, id, x, y, w, h, color, onButtonEvent )
    local group = newGroup(parent)
    newRect(group, 0, 0, w, h, color)
    newText(group, id, 0, 0, 0, 0, native.systemFont, 50, {0}, "center")
    group.x = x
    group.y = y
    group.id = id
    group:addEventListener("tap", onButtonEvent)
    return group
end

------------------------------------------------------------------
-- newTableView
------------------------------------------------------------------
function newTableView( parent, x, y, w, h, onRowRender )
    local tableView = widget.newTableView
    {
        x = x,
        y = y,
        width = w,
        height = h,
        hideBackground = false,
        onRowRender = onRowRender,
        onRowTouch = nil,
        listener = __on_table_listener,
        maxVelocity = 5.2,
        friction = 0.972,
        rowTouchDelay = 0,
        noLines = true,
        hideScrollBar = true,
    }

    if parent then
        parent:insert(tableView)
    end

    function tableView:getPos()
        return tableView._view.y
    end

    function tableView:setPos( y )
        tableView._view.y = y
        tableView:scrollToY({0, 0})
    end

    function tableView:scroll( index )
        tableView._prevPos = nil
        tableView:scrollToIndex( index, 300 )
    end

    function tableView:scrollY( y )
        tableView._prevPos = nil
        tableView:scrollToY( { y=y, time=300 } )
    end

    return tableView
end

------------------------------------------------------------------
-- MessageBox
------------------------------------------------------------------
function MessageBox(msg, fontSize, panColor, textColor)
    local delay = 1500
    local group = newGroup()

    if fontSize == nil then fontSize = 50 end
    if panColor == nil then panColor = {0,0,0,0.6} end
    if textColor == nil then textColor = {1} end

    local emptyH = 120

    local panel = newRect(group, 0, 0, orgW, 1, panColor)
    local text = newText(group, msg, 0, 0, orgW-120, 0, native.systemFont, fontSize, textColor, "center")

    panel.height = text.height + 120 + emptyH
    panel.y = panel.height*0.5
    text.y = panel.y + (emptyH*0.5)

    group.x = centerX
    group.y = -group.height

    transition.to( group, { time=200, y=0, transition=easing.outQuad } )
    transition.to( group, { delay=200+delay, time=300, y=-panel.height, transition=easing.outQuad, 
        onComplete=function()
            display.remove( group )
            group = nil
        end
    } )
end
